import datetime
import pydantic
import typing
import sipametrics.models.base as base_models
import sipametrics.constants as CONSTANTS


class ColumnAndValue(base_models.CamelCaseBaseModel):
    column: str
    value: str

class Constraint(base_models.CamelCaseBaseModel):
    criteria: list[typing.Union["EqualityConstraint", "RangeConstraint", ColumnAndValue, "Constraint"]] = pydantic.Field(min_length=1)
    type: CONSTANTS.CriteriaAggregationType = CONSTANTS.CriteriaAggregationType.AND
    start_date: typing.Optional[datetime.date] = None

    def to_dict(self) -> dict:
        payload = self.model_dump(mode="json", by_alias=True, exclude_defaults=True)
        return payload

class EqualityConstraint(Constraint):
    target: float

class RangeConstraint(Constraint):
    lower: float
    upper: float

class CustomBenchmarkRequest(base_models.CamelCaseBaseModel):
    external_ticker: str
    start_date: typing.Optional[datetime.date] = None
    end_date: typing.Optional[datetime.date] = None
    constraints: typing.Optional[list[typing.Union[Constraint, EqualityConstraint, RangeConstraint]]] = None
    initial_index_value: typing.Optional[float] = None
    allocations: typing.Optional[bool] = None
    risk_profiles: typing.Optional[bool] = None
    include_constituents: typing.Optional[bool] = None

    def to_dict(self) -> dict:
        payload = self.model_dump(mode="json", by_alias=True, exclude={"constraints"}, exclude_defaults=True)
        if self.constraints:
            constraints = []
            for constraint in self.constraints:
                constraints.append(constraint.to_dict())
            payload["constraints"] = constraints

        return payload


import pydantic, pydantic.alias_generators


class CamelCaseBaseModel(pydantic.BaseModel):
    model_config = pydantic.ConfigDict(alias_generator=pydantic.alias_generators.to_camel, populate_by_name=True, extra="forbid")


import datetime
import typing
import sipametrics.models.base as base_models


class FundDatapoint(base_models.CamelCaseBaseModel):
    contribution: float
    distribution: float
    date: datetime.date

class FundData(base_models.CamelCaseBaseModel):
    cashflows: list[FundDatapoint]
    final_nav: float
    fund_name: typing.Optional[str]

class IndexBenchmarkDatapoint(base_models.CamelCaseBaseModel):
    level: float
    date: datetime.date

class IndexBenchmarkData(base_models.CamelCaseBaseModel):
    index_benchmark_data: list[IndexBenchmarkDatapoint]
    index_name: typing.Optional[str] = None

class IndexBenchmarkTicker(base_models.CamelCaseBaseModel):
    ticker: str
    index_name: typing.Optional[str] = None

class FundAlphaData(base_models.CamelCaseBaseModel):
    fund_data: FundData
    market_index_data: typing.Union[IndexBenchmarkData, IndexBenchmarkTicker]
    benchmark_data: typing.Optional[typing.Union[IndexBenchmarkData, IndexBenchmarkTicker]] = None
    weight: typing.Optional[float] = None

class AlphaRequest(base_models.CamelCaseBaseModel):
    fund_alpha_data: list[FundAlphaData]

    def to_dict(self):
        return self.model_dump(mode="json", by_alias=True, exclude_defaults=True)


import typing
import sipametrics.models.base as base_models
import sipametrics.constants as CONSTANTS


class IndicesCatalogueRequest(base_models.CamelCaseBaseModel):
    product: CONSTANTS.Product
    app: typing.Optional[CONSTANTS.App] = None

    def to_dict(self) -> dict:
        payload:dict[str, typing.Union[str, int]] = { "product": self.product.value if isinstance(self.product, CONSTANTS.Product) else self.product }
        if self.app:
            if self.app == CONSTANTS.App.MARKET_INDICES:
                app_key = "marketIndices" if self.product == CONSTANTS.Product.PRIVATE_EQUITY else "indexApp"
                payload[app_key] = 1
            elif self.app == CONSTANTS.App.THEMATIC_INDICES:
                app_key = "peccsBenchmarks" if self.product == CONSTANTS.Product.PRIVATE_EQUITY else "assetValuation"
                payload[app_key] = 1

        return payload


# PUBLIC_API_URL = "https://api.sipametrics.com"
PUBLIC_API_URL = "https://beta-backend.edhecinfra.institute"
URLS = {
    "metrics": f"{PUBLIC_API_URL}/data-service/v2/queries/metrics",
    "infra_equity_comparable": f"{PUBLIC_API_URL}/compute-service/v2/pi-comparables/average/by/profiles",
    "infra_debt_comparable": f"{PUBLIC_API_URL}/compute-service/v2/pi-comparables/average/by/profiles",
    "private_equity_comparable": f"{PUBLIC_API_URL}/compute-service/v2/pe-comparables/average/by/profiles",
    "private_equity_comparable_boundaries": f"{PUBLIC_API_URL}/compute-service/v2/pe-comparables/boundaries/by/profiles",
    "term_structure": f"{PUBLIC_API_URL}/data-service/v2/yields",
    "indices_catalogue": f"{PUBLIC_API_URL}/data-service/v2/lists/indices",
    "taxonomies": f"{PUBLIC_API_URL}/data-service/v2/lists/taxonomy/{{taxonomy}}",
    "private_infra_custom_benchmarks": f"{PUBLIC_API_URL}/compute-service/v2.1/pi-custom-index/metrics",
    "private_equity_custom_benchmarks": f"{PUBLIC_API_URL}/compute-service/v2.1/pe-custom-index/metrics",
    "private_equity_region_tree": f"{PUBLIC_API_URL}/compute-service/v2/pe-comparables/region-tree",
    "direct_alpha": f"{PUBLIC_API_URL}/compute-service/v1/alpha/compute"
}
VERSION = "v2"
SOURCE = "privateMetrics-python-sdk"